import { useRef, useMemo } from 'react';
import { Vector3 } from 'three';
import { useFrame } from 'react-three-fiber';
import vertority from '../utils/vertority';
import { computeBoundingbox } from '../utils/element';
import random from '../utils/random';
import { deg2rad } from '../utils/angle';
import point from '../utils/point';
import { getCoord } from '../utils/scene';
import { DIRS } from '../utils/constants';
var RAIN_COLORS = ['#fff', '#0F203B'];
/**
 * 角度正负数值
 * @param {number | undefined} angle
 */

var angle2dir = function angle2dir(angle) {
  if (!angle || angle === 0) {
    return 0;
  }

  return angle / Math.abs(angle);
};

var angle2placement = function angle2placement(angle) {
  if (!angle || angle === 0) {
    return 'fromTop';
  }

  return angle < 0 ? 'fromRight' : 'fromLeft';
};

export var DEFAULT_RAINPROPS = {
  count: 100,
  angle: -45
};
export var useHaze = function useHaze() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_RAINPROPS,
      _ref$angle = _ref.angle,
      angle = _ref$angle === void 0 ? -45 : _ref$angle,
      _ref$count = _ref.count,
      count = _ref$count === void 0 ? 100 : _ref$count;

  // hazedrop start position data
  var _angle = deg2rad(angle);

  var startpoints = useRef({
    fromTop: new Vector3().fromArray(point.axisxy.top),
    fromRight: new Vector3().fromArray(point.axisxy.right),
    fromLeft: new Vector3().fromArray(point.axisxy.left)
  }).current; // radindrop come from which orientation

  var comefrom = useRef({
    fromLeft: ['fromTop', 'fromRight'],
    fromRight: ['fromTop', 'fromLeft'],
    fromTop: ['fromTop']
  }).current;
  var lines = useMemo(function () {
    return Array(count).fill(0).map(function () {
      // h / deltax = tan(ang)
      // 直角边
      var leg = 0.5; // Math.random() * 2
      // FIXME: should modify from angle

      var orientation = random.inRange(comefrom[angle2placement(_angle)]); // noise vertor for startpoint

      var vertor = vertority.fromPlacement(orientation).add(new Vector3(0, 0, point.fromAxisZ()));
      return {
        vertices: [// endpoint
        new Vector3().copy(startpoints[orientation]).add(vertor) // distance vertor
        .add(new Vector3(leg / Math.tan(_angle), leg, 0)), // startpoint
        new Vector3().copy(startpoints[orientation]).add(vertor)],
        angle: _angle,
        leg: leg,
        opacity: Math.random() + 0.5,
        orientation: orientation,
        color: random.inRange(RAIN_COLORS),
        dashArray: Math.random() * 0.3
      };
    });
  }, [_angle, count]);
  return {
    lines: lines
  };
};
export var useHazeDrop = function useHazeDrop(hazedrop, mat, props) {
  var _a;

  var friction = useRef(random.inRange(DIRS) === 1 ? 0 : 1 * 0.01 * Math.round(Math.random() * 10)).current;
  var coord = useRef(getCoord()).current;
  var vy0 = useRef(0.001 + friction); // vy0 / vx0 = tan(angle)

  var vx0 = useRef(0.001 + friction);
  var a = useRef(0.002);

  var _computeBoundingbox = computeBoundingbox((_a = props) === null || _a === void 0 ? void 0 : _a.value.vertices[0]),
      offsetTop = _computeBoundingbox.offsetTop;

  useFrame(function () {
    var _a, _b;

    if (((_a = hazedrop.current) === null || _a === void 0 ? void 0 : _a.position.y) === undefined || !((_b = props) === null || _b === void 0 ? void 0 : _b.value) || !mat.current) {
      return;
    } // hazedrop加速下落


    hazedrop.current.position.y -= vy0.current; // left hazedrop从左到右移动, right hazedrop从右到左移动

    hazedrop.current.position.x -= vx0.current * angle2dir(props.value.angle); // 从无到有比较真实

    vy0.current += a.current;
    vx0.current += a.current; // 判断hazedrop是否出了边界

    if (offsetTop + Math.abs(hazedrop.current.position.y) > coord[1] * 2 + props.value.leg) {
      var vertor = vertority.fromPlacement(props.value.orientation); // 随机hazedrop初始位置, 避免loop重复

      hazedrop.current.position.set(vertor.x, vertor.y, vertor.z);
      vy0.current = 0.0001;
      vx0.current = 0.0001;
    }
  });
};
//# sourceMappingURL=use-haze.js.map
